@@ -51,7 +51,8 @@ class RevocationChecker(object):
 
             # New versions of openssl want -header var=val, old ones want -header var val
             test_host_format = Popen(["openssl", "ocsp", "-header", "var", "val"],
-                                     stdout=PIPE, stderr=PIPE, universal_newlines=True)
+                                     stdout=PIPE, stderr=PIPE, universal_newlines=True,
+                                     env=util.env_no_snap_for_external_calls())
             _out, err = test_host_format.communicate()
             if "Missing =" in err:
                 self.host_args = lambda host: ["Host=" + host]
